<# 
           .SYNOPSIS  
           This script will pullall, drain or balance cluster resources from the command line. 
 
           .DESCRIPTION 
           The script will pullall, drain or balance cluster resources from the command line. 
		   
		   .NOTES 
           File Name : ClusterResourceMover.ps1
           Author   : Mark Harris
           Requires  : Windows Server 2012 R2 PowerShell
                       Windows Server 2012 R2 Clustering Cmdlets 
                       Windows Server 2008 R2 PowerShell
					   Windows Server 2008 R2 Clustering Cmdlets
					   Windows Server 2016 PowerShell
                       Windows Server 2016 Clustering Cmdlets 					   
           Version   : 1.0 (June 26 2017)
					       
           .PARAMETER drain or pullall or balance
           Specifies send, retrieve or delegate cluster resources 
 
           .INPUTS 
           ClusterResourceMover accepts drain, pullall and balance 
 
           .OUTPUTS 
           Log to c:\support directory and log to Application EventLog and sends email to teams or individuals. 
 
           .EXAMPLE 
           C:\PS> .\ClusterResourceMover drain # drains all cluster resources to the other node. 
			
		   .EXAMPLE 
           C:\PS> .\ClusterResourceMover pullall # Retrives all cluster resources to the current node.
		   
		   .EXAMPLE 
           C:\PS> .\ClusterResourceMover balance # Moves resource to preferred owner node.
		   
#> 
# Get User input and execute function Must be first line in code

Param([Parameter(Mandatory=$true,HelpMessage="Enter in drain, pullall, balance")][String]$ClusOp)

# Create EventLog Source, will not create if exists
$Result = test-path -path 'HKLM:\SYSTEM\ControlSet001\Services\EventLog\Application\ClusterHealth'
if ($Result -eq "True")
	{ Write-host "No changes made to registry, all good" }
Else {
	New-EventLog -Source "ClusterHealth" -LogName "Application" -MessageResourceFile "%SystemRoot%\System32\EventCreate.exe"
	}

# Create Local logfile
Import-Module FailoverClusters
$node = hostname
$date1=Get-Date -Format "yyyy-MM-dd hh:mm tt"
$clustervip=Get-Cluster 
$clustervipname=($clustervip.Name)
$admin = (Get-Content -Path env:username).ToString()
$LogFileName = "c:\Support\ClusterResMove-"+ $node +".txt"
Out-File -append -FilePath $LogFileName -Encoding ASCII
Add-Content -Path $LogFileName -Value "Computer: $node" -Encoding ASCII
Add-Content -Path $LogFileName -Value "*****************************" -Encoding ASCII

Function Balance {
Write-EventLog -LogName "Application" -Source "ClusterHealth" -EventID 101 -EntryType Information -Message "($clustervipname) Cluster Balance Operation Started for Windows Patching"
Import-Module FailoverClusters
Add-Content -Path $LogFileName -Value "Date: $date1" -Encoding ASCII
Add-Content -Path $LogFileName -Value "Executed By: $admin" -Encoding ASCII
Add-Content -Path $LogFileName -Value "Current Cluster Assignments" -Encoding ASCII
$computer = get-content env:computername
$computer = $computer.ToLower()

Get-ClusterGroup | Out-File -append  -Encoding ASCII -FilePath $LogFileName

$clustergroups = Get-ClusterGroup | Where-Object {$_.IsCoreGroup -eq $false}
foreach ($cg in $clustergroups)
{
    $CGName = $cg.Name
    Add-Content -Path $LogFileName -Value "Getting Cluster Group Owners" -Encoding ASCII
    $CurrentOwner = $cg.OwnerNode.Name
    $POCount = (($cg | Get-ClusterOwnerNode).OwnerNodes).Count
    if ($POCount -eq 0)
    {
        Add-Content -Path $LogFileName -Value "Info: $CGName doesn't have a preferred owner!" -Encoding ASCII
    }
    else
    {
        $PreferredOwner = ($cg | Get-ClusterOwnerNode).Ownernodes[0].Name
		Add-Content -Path $LogFileName -Value "$PreferredOwner" -Encoding ASCII
        if ($CurrentOwner -ne $PreferredOwner)
        {
            Add-Content -Path $LogFileName -Value "Moving resource to $PreferredOwner" -Encoding ASCII
            $cg | Move-ClusterGroup -Node $PreferredOwner
        }
        else
        {
            Add-Content -Path $LogFileName -Value "Resource is already on preferred owner! ($PreferredOwner)"  -Encoding ASCII
        }
    }
}
  Add-Content -Path $LogFileName -Value "Cluster Balance Operation Completed" -Encoding ASCII
  Add-Content -Path $LogFileName -Value "New Cluster Assignments" -Encoding ASCII
  Get-ClusterGroup | Out-File -append  -Encoding ASCII -FilePath $LogFileName
  Write-EventLog -LogName "Application" -Source "ClusterHealth" -EventID 201 -EntryType Information -Message "($clustervipname) Cluster Balance Operation completed for Windows Patching"
  # Uncomment FuncMail below to use email sending option uncomment for P&C clusters.
  # FuncMail -To "destinationemail@me.com" -From "frommail@me.com"  -Subject "($clustervipname) Cluster Balance Opertaion completed by ($node)." -Body "($clustervipname) Cluster Balance Operation completed for Windows Patching" -smtpServer "smtpmailserver"
}

Function Drain {
Write-EventLog -LogName "Application" -Source "ClusterHealth" -EventID 101 -EntryType Information -Message "Cluster Drain Operation started for Windows Patching"
Import-Module FailoverClusters
Add-Content -Path $LogFileName -Value "Date: $date1" -Encoding ASCII
Add-Content -Path $LogFileName -Value "Executed By: $admin" -Encoding ASCII
Add-Content -Path $LogFileName -Value "Current Cluster Assignments" -Encoding ASCII
Get-ClusterGroup | Out-File -append  -Encoding ASCII -FilePath $LogFileName 
$computer = get-content env:computername
$computer = $computer.ToLower()
$destnode = Get-clusternode | select Name

# Convert to string for use in foreach-object
[string]$drainnode = ($destnode.Name -ne $computer) 

Get-ClusterGroup |
foreach-object `
	{
	If ($_.Name -ne $computer)
		{
		Move-ClusterGroup -Name $_.Name -Node $drainnode
		}
	
	}
	
Add-Content -Path $LogFileName -Value "($clustervipname) Drain Operation Completed to $drainnode " -Encoding ASCII
Add-Content -Path $LogFileName -Value "New Cluster Assignments" -Encoding ASCII
Get-ClusterGroup | Out-File -append  -Encoding ASCII -FilePath $LogFileName
Write-EventLog -LogName "Application" -Source "ClusterHealth" -EventID 201 -EntryType Information -Message "($clustervipname) Cluster Drain Operation completed for Windows Patching"
# Uncomment FuncMail below to use email sending option uncomment for P&C clusters.
# FuncMail -To "destinationemail@me.com" -From "frommail@me.com"  -Subject "($clustervipname) Cluster Drain Opertaion completed by ($node)." -Body "($clustervipname) Cluster Drain Operation completed for Windows Patching" -smtpServer "smtpmailserver"
}

Function PullAll {
Write-EventLog -LogName "Application"  -Source "ClusterHealth" -EventID 101 -EntryType Information -Message "Cluster PullAll Operation Started for Windows Patching"
Import-Module FailoverClusters
Add-Content -Path $LogFileName -Value "Date: $date1" -Encoding ASCII
Add-Content -Path $LogFileName -Value "Executed By: $admin" -Encoding ASCII
Add-Content -Path $LogFileName -Value "Current Cluster Assignments" -Encoding ASCII
Get-ClusterGroup | Out-File -append  -Encoding ASCII -FilePath $LogFileName 
$computer = get-content env:computername
$computer = $computer.ToLower()
Get-clusternode | Get-ClusterGroup |
foreach-object `
	{
	
	If ($_.Name -ne $computer)
		{ 
		Move-ClusterGroup -Name $_.Name -Node $computer
		}
	}
Add-Content -Path $LogFileName -Value "($clustervipname) PullAll Operation Completed to $computer " -Encoding ASCII
Add-Content -Path $LogFileName -Value "New Cluster Assignments" -Encoding ASCII
Get-ClusterGroup | Out-File -append  -Encoding ASCII -FilePath $LogFileName
Write-EventLog -LogName "Application" -Source "ClusterHealth" -EventID 201 -EntryType Information -Message "($clustervipname) Cluster PullAll Operation completed for Windows Patching"
# Uncomment FuncMail below to use email sending option uncomment for P&C clusters.
# FuncMail -To "destinationemail@me.com" -From "frommail@me.com"  -Subject "($clustervipname) Cluster PullAll Opertaion completed by ($node)." -Body "($clustervipname) Cluster PullAll Operation completed for Windows Patching" -smtpServer "smtpmailserver"
}

# Take the parameter and validate the input and call the functions.
If ("drain","pullall","balance" -NotContains $ClusOp) 
  { 
    Throw "Not a valid option! Please use drain, pullall or balance option" | Out-File -append  -Encoding ASCII -FilePath $LogFileName 
  } 

function FuncMail {
    #param($strTo, $strFrom, $strSubject, $strBody, $smtpServer)
    param($To, $From, $Subject, $Body, $smtpServer)
    $msg = new-object Net.Mail.MailMessage
    $smtp = new-object Net.Mail.SmtpClient($smtpServer)
    $msg.From = $From
    $msg.To.Add($To)
    $msg.Subject = $Subject
    $msg.IsBodyHtml = 1
    $msg.Body = $Body
    $smtp.Send($msg)
}
  
 # All parameters are valid call function 
If ($ClusOp -eq "drain") { Drain }
If ($ClusOp -eq "pullall") { PullAll }
If ($ClusOp -eq "balance") { Balance }
